#!/usr/bin/python

from nameparser import HumanName
import MySQLdb
import random
import mechanize
import string
import json
import locale
import datetime
import urllib
import time
import socks
import socket
import numpy
from statistics import variance, mean, pvariance, stdev

###############################################################################
#  CONSTANTS
###############################################################################
start_year = 1870
end_year   = 2017
output_dir = 'datasets'

###############################################################################
#  CLASSES/METHODS
###############################################################################
def output_gnuplot(filename, series):
    # Open the file
    f = open(filename, 'w')

    # Iterate over all elements in the series
    for i in series:
        # Print the series 
        for j in i:
            f.write(str(j)+' ')
        f.write('\n')

class aps_article:
    def __init__(self):
        # Initialize all of the class attributes to sane values
        self.title     = ''
        self.authors   = []
        self.pub_date  = datetime.date(1857,1,1)
        self.pdf_link  = ''
        self.citation_count = int()
        self.doi       = ''
        self.journal   = ''
        self.vol       = int()
        self.page      = ''
        self.pub_type  = ''
        self.aps_id    = ''
        self.aps_score = float()
        self.references = []

    def clear(self):
        self.title     = ''
        self.authors   = []
        self.pub_date  = datetime.date(1857,1,1)
        self.pdf_link  = ''
        self.citation_count = int()
        self.doi       = ''
        self.journal   = ''
        self.vol       = int()
        self.page      = ''
        self.pub_type  = ''
        self.aps_id    = ''
        self.aps_score = float()
        self.query_str = int()
    
    def init_from_json(self, json_obj):
        ## Grab all of the strings from the JSON object
        # Get the article's title
        try:
            self.title    = json_obj['title']
        except:
            raise NameError('could not retrieve title from JSON object')

        # Get the DOI
        try:
            self.doi      = json_obj['doi']
        except:
            self.doi      = ''

        # Get the journal it was published in
        try:
            self.journal  = json_obj['journal']
        except:
            self.journal  = ''

        # Get the publication type
        try:
            self.pub_type = json_obj['type']
        except:
            self.pub_type = ''

        # Get the APS ID
        try:
            self.aps_id   = json_obj['id']
        except:
            self.aps_id   = ''

        # Get a link to the PDF
        try:
            self.pdf_link = 'http://journals.aps.org'+json_obj['actions']['pdf']['link']
        except:
            self.pdf_link = ''

        ## Get anything that uses a simple conversion
        # Get the publication date
        try:
            self.pub_date = datetime.datetime.strptime(json_obj['date'], "%Y-%m-%d")
        except:
            self.pub_date  = datetime.date(1857,1,1)

        # Get the APS score
        try:
            self.aps_score = json_obj['score']
        except:
            self.aps_score = 0.0

        ## Get the volume and page number
        try:
            # Retrieve the citation string
            citation = json_obj['citation']

            # Get the location of the bold tags in it
            begin = string.find(citation, '<b>') + 3
            end   = string.find(citation, '</b>')

            # Convert the number between them into the volume number
            self.vol = int(citation[begin:end].replace(',', ''))

            # The page number begins 5 chars after the end of the volume and ends with the year
            begin = end + 6
            end   = string.find(citation, ' (', begin)

            # Copy the page number
            self.page = citation[begin:end].replace(',', '')
        except:
            self.page = ''
            self.vol = 0

        ## Get the number of citations
        # Check if there is a citation count
        try:
            # Get the test from the object formatted as %d citations
            citation_count = json_obj['citation_count_text']

            # Get the point at which the word citations is written
            end =  string.find(citation_count, ' citation')

            # Convert the number of citations
            self.citation_count = int(citation_count[:end].replace(',', ''))
        except:
            self.citation_count = 0

        ## Get the author list
        try:
            # Split the list around ' and '
            and_split = json_obj['authors'].split(' and ')

            # Go through and split the rest around commas
            for i in and_split:
                self.authors.extend(i.split(', '))
        except:
            self.authors = []

        ## Get the list of references
        try:
            print 'Finding references'

            # Get the URL for the references page
            url = self.pdf_link.replace('pdf', 'references')
            
            # Instantiate our browser
            browser = mechanize.Browser()

            # Set some parameters from the browser
            browser.set_handle_robots(False)
            browser.addheaders = [('User-agent', 'Firefox')]

            # Sleep for a random ammount of time
            time.sleep(abs(random.gauss(2.0, 1.5)))

            # Load the references page
            browser.open(url)

            # Verify that the page opened
            assert browser.viewing_html()

            # Get the response
            query_result = browser.response().read()

            # Iterate over each reference
            count = 1
            while(True):
                # Find the citation here
                begin = query_result.find('<li id="c'+str(count)+'">') + 12

                # Check if it wasn't found
                if(begin == 11):
                    break

                # Get the end of the citation
                end   = query_result.find('</li>', begin)
        
                # Increment the counter
                count += 1

                self.references.append(query_result[begin:end])
        except:
            print 'failed to find references'
            return


    def has_author(self, author):
        return author in self.authors

    def print_csv(self):
        print(self.str)

def aps_search_url(author_queries, page_num):
    # Write the base of the URL to the return string
    url = 'http://journals.aps.org/search/results?page='
    url += str(page_num)
    url += '&date=&sort=relevance&per_page=100&start_date=&end_date=&clauses='

    # Add the first search clause to the URL
    url += '[{%22operator%22:%22AND%22,%22field%22:%22author%22,%22value%22:%22'+urllib.quote_plus(author_queries[0])+'%22}'

    # Add each remaining author as a clause to the string
    for author in author_queries[1:]:
        url += ',{%22operator%22:%22OR%22,%22field%22:%22author%22,%22value%22:%22'+urllib.quote_plus(author)+'%22}'

    # End the search string and return it
    url += ']'
    return url

def aps_author_name(in_name):
    # Get the name structure
    name = HumanName(in_name)

    # Convert to initials
    query_str = ''
    if(not name.middle):
        query_str = name.first[0]+'. '+name.last
    else:
        query_str = name.first[0]+'. '+name.middle[0]+'. '+name.last

    return query_str

# Compare if two potentially initialized words could be the same
def initial_comp(init1, init2):
    # If either are initialized compare the first letter
    if( (len(init1) == 2 and init1[1] == '.') or (len(init2) == 2 and init2[1] == '.') ):
        return (init1[0] == init2[0])
    else:
        return (init1 == init2)

# Determine whether or not two authors are the same including initialization
def author_comp(auth1, auth2):
    # Get name structures for them
    auth1_name = HumanName(auth1)
    auth2_name = HumanName(auth2)

    # Determine if the first name of either is initialized
    if(auth1_name.first and auth2_name.first):
        if(not initial_comp(auth1_name.first, auth2_name.first)):
            return False
    if(auth1_name.middle and auth2_name.middle):
        if(not initial_comp(auth1_name.middle, auth2_name.middle)):
            return False
    if(auth1_name.last and auth2_name.last):
        if(not initial_comp(auth1_name.last, auth2_name.last)):
            return False

    # They matched
    return True
    
def author_in_list(m_list, author):
    for i in m_list:
        if(author_comp(i, author)):
            return True
    return False

def ending_square_brace(m_str, begin):
    level = 0
    count = begin
    for i in m_str[begin:]:
        if(i == '['):
            level += 1
        if(i == ']'):
            level -= 1
        if(level == 0):
            return count
        count += 1

# Datatype for storing course catalog entries
class course_catalog_entry:
    # Initialize all of the member variables
    def __init__(self):
        self.date   = datetime.date(1857,1,1)
        self.name   = ''
        self.degree = ''
        self.title  = ''

    def set(self, date, name, degree, title):
        self.date   = date
        self.name   = name
        self.degree = degree
        self.title  = title

    def get(self, date, name, degree, title):
        date = self.date
        name = self.name
        degree = self.degree
        title = self.title

class aps_article_entry:
    def __init__(self):
        self.article_id     = 0
        self.title          = ''
        self.pub_date       = datetime.date(1857,1,1)
        self.pdf_link       = ''
        self.citation_count = 0
        self.doi            = ''
        self.journal        = ''
        self.vol            = 0
        self.page           = ''
        self.pub_type       = ''
        self.aps_id         = 0
        self.aps_score      = 0.0
        self.query_id       = 0
        self.authors        = []
        self.refs           = []

class database_interface:
    def __init__(self):
        self.db = ''

    def connect(self):
        self.db = MySQLdb.connect("localhost", "root", "dummy_password", "history_of_physics")
        
    def drop_aps_tables(self):
        # Get a cursor
        cursor = self.db.cursor()

        # Drop the aps_tables
        try:
            cursor.execute('DROP TABLE aps_article_metadata')
        except:
            hey = 'ho'

        try:
            cursor.execute('DROP TABLE aps_author_association')
        except:
            hey = 'ho'

        try:
            cursor.execute('DROP TABLE aps_query_history')
        except:
            hey = 'ho'

        try:
            cursor.execute('DROP TABLE aps_references')
        except:
            hey = 'ho'

        # Commit the changes
        self.db.commit()
        
    def create_aps_tables(self):
        # Get a cursor
        cursor = self.db.cursor()

        # Create the article metadata table
        cursor.execute("""CREATE TABLE aps_article_metadata (article_id INT NOT NULL AUTO_INCREMENT, title varchar(512) NOT NULL, pub_date DATE, pdf_link VARCHAR(512), citation_count INT, doi VARCHAR(512), journal VARCHAR(256), vol INT, page VARCHAR(128), pub_type VARCHAR(128), aps_id VARCHAR(128), aps_score DOUBLE, query_id INT, PRIMARY KEY (article_id))""")

        # Create the author table
        cursor.execute("""CREATE TABLE aps_author_association (id INT NOT NULL AUTO_INCREMENT, name VARCHAR(512), article_id INT NOT NULL, PRIMARY KEY (id))""")

        # Create the query table
        cursor.execute("""CREATE TABLE aps_query_history (query_id INT NOT NULL AUTO_INCREMENT, query_str VARCHAR(512), valid BOOLEAN NOT NULL, PRIMARY KEY (query_id))""")

        # Create the references table
        cursor.execute("""CREATE TABLE aps_references (id INT NOT NULL AUTO_INCREMENT, article_id INT NOT NULL, reference VARCHAR(1024), PRIMARY KEY (id))""")

        # Commit the changes
        self.db.commit()

    def query_author(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Check in the DB that we aren't repeating a query
        cursor.execute("""SELECT * FROM aps_query_history WHERE query_str = %s""", [query_str])
        if(cursor.rowcount > 0):
            print 'Query "'+query_str+'" has already been run, returning.'
            return 
        
        # Print what we are doing
        print('Running query "'+query_str+'"')

        # Get a reduced query string for our author
        red_query_str = aps_author_name(query_str)

        if(True):
            # Instantiate our browser
            browser = mechanize.Browser()

            # Set some parameters from the browser
            browser.set_handle_robots(False)
            browser.addheaders = [('User-agent', 'Firefox')]

            # Initialize a counter for the page number
            page_num = 1

            # Iterate over the pages that have valid data
            # Also, python doesn't have a do-while loop????
            articles = []
            while True:
                # Print where we are in the loop
                print("Reading metadata from page "+str(page_num))

                # Sleep for a random ammount of time
                time.sleep(abs(random.gauss(5.0, 1.5)))

                # Load the aps search page
                url = aps_search_url([red_query_str], page_num)
                browser.open(url)

                # Verify that the page opened
                assert browser.viewing_html()

                # Get the response
                query_result = browser.response().read()

                # The string we want is between the square brackets between the phrases listed below
                begin = string.find(query_result, '}}},"results":[{"') + 14
                end   = ending_square_brace(query_result, begin) + 1

                # Decode the JSON string into a dictionary object
                try:
                    returned_articles = json.loads(query_result[begin:end])
                except:
                    # We are on a page without data, IE the last page+1
                    break
        
                # Iterate over the JSON data for each article using them to instantiate the articles
                for article in returned_articles:
                    # Initialize the article with the JSON data collected from the APS website
                    tmp = aps_article()
                    tmp.init_from_json(article)

                    # Compare the authors to the query string to make sure that our reduced string didn't find erroneous results
                    # Add the article to our list of articles
                    if(author_in_list(tmp.authors, query_str)):
                        articles.append(tmp)

                # Increment the counter
                page_num += 1

        #except:
            # Report the error and fail
           # print('ERROR: Query "'+query_str+'" failed')
           # raise NameError('Query "'+query_str+'" failed')

        # List what we are doing
        print 'Pushing query results to database'

        # Begin by pushing our search history to the database
        query_id = int()
        try:
            cursor.execute("""INSERT INTO aps_query_history (query_str, valid) VALUES (%s, %s)""", [query_str, '0'])
            query_id = int(cursor.lastrowid)
        except:
            raise NameError('could not insert queries into DB')
            
        # Push the articles to the DB
        article_ids = []
        for i in articles:
            try:
                cursor.execute("""INSERT INTO aps_article_metadata (title, pub_date, pdf_link, citation_count, doi, journal, vol, page, pub_type, aps_id, aps_score, query_id) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)""", [i.title, i.pub_date.isoformat(), i.pdf_link, i.citation_count, i.doi, i.journal, i.vol, i.page, i.pub_type, i.aps_id, i.aps_score, query_id])
                i.db_id = int(cursor.lastrowid)
            except:
                raise NameError('could not insert article into DB')

        # Push the author lists to the DB
        for i in articles:
            for j in i.authors:
                cursor.execute("""INSERT INTO aps_author_association (name, article_id) VALUES (%s, %s)""", [j, i.db_id])

        # Push the reference lists to the DB
        for i in articles:
            for j in i.references:
                cursor.execute("""INSERT INTO aps_references(article_id, reference) VALUES (%s, %s)""", [i.db_id, j])

        # Update the table that this query is now valid as all of the data has been pushed
        cursor.execute("""UPDATE aps_query_history SET valid=1 WHERE query_id=%s""", [query_id])

        # Commit the changes to the database
        print 'Committing changes to database'
        self.db.commit()

    # Routine to clean invalid data from the database
    def clear_invalid_data(db):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Set the binlog format to mixed to avoid problems with the next few statements
        cursor.execute("""SET SESSION binlog_format=MIXED""")

        # Delete all rows from aps_query_history where the query is invalid
        cursor.execute("""DELETE FROM aps_query_history WHERE valid IS NULL OR valid='0' """)

        # Delete all of the articles not corresponding to a query in aps_article_metadata
        cursor.execute("""DELETE FROM aps_article_metadata WHERE query_id NOT IN (SELECT query_id FROM aps_query_history)""")

        # Delete all of the authors not corresponding to an article in aps_author_association
        cursor.execute("""DELETE FROM aps_author_association WHERE article_id NOT IN (SELECT article_id FROM aps_article_metadata)""")

        # Delete all of the reference data not corresponding to an article
        cursor.execute("""DELETE FROM aps_references WHERE article_id NOT IN (SELECT article_id FROM aps_article_metadata)""")

        # Commit the changes
        self.db.commit()

    # Get the course catalog entries from a particular query string
    def get_course_catalog_entries(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute("""SELECT * FROM course_catalog_faculty_entries WHERE """+query_str)

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()

            # Initiailze a course catalog entry and set it
            m_entry = course_catalog_entry()
            m_entry.set(row[1], row[2], row[3], row[4])

            # Append the result converting to the correct datatype
            rows.append(m_entry)

        # Return the rows
        return rows

    def get_article_entries(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute("""SELECT * FROM aps_article_metadata WHERE """+query_str)

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()

            # Initiailze a course catalog entry and set it
            m_entry = aps_article_entry()

            # Set all of the entries
            m_entry.article_id     = row[0]
            m_entry.title          = row[1]
            m_entry.pub_date       = row[2]
            m_entry.pdf_link       = row[3]
            m_entry.citation_count = row[4]
            m_entry.doi            = row[5]
            m_entry.journal        = row[6]
            m_entry.vol            = row[7]
            m_entry.page           = row[8]
            m_entry.pub_type       = row[9]
            m_entry.aps_id         = row[10]
            m_entry.aps_score      = row[11]
            m_entry.query_id       = row[12]

            # Query authors
            loop_cursor = self.db.cursor()
            loop_cursor.execute("""SELECT * FROM aps_author_association WHERE article_id="""+str(row[0]))

            for x in xrange(0, loop_cursor.rowcount):
                loop_row = loop_cursor.fetchone()

                # Add in all of the authors
                m_entry.authors.append(loop_row[1])

            # Query references
            loop_cursor = self.db.cursor()
            loop_cursor.execute("""SELECT * FROM aps_references WHERE article_id="""+str(row[0]))

            for x in xrange(0, loop_cursor.rowcount):
                loop_row = loop_cursor.fetchone()

                # Add in all of the authors
                m_entry.refs.append(loop_row[2])


            # Append the result converting to the correct datatype
            rows.append(m_entry)

        # Return the rows
        return rows

    def get_article_entries_fac(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute("""SELECT * FROM aps_article_metadata WHERE article_id in (SELECT DISTINCT article_id FROM aps_author_association WHERE """+query_str+""")""")

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()

            # Initiailze a course catalog entry and set it
            m_entry = aps_article_entry()

            # Set all of the entries
            m_entry.article_id     = row[0]
            m_entry.title          = row[1]
            m_entry.pub_date       = row[2]
            m_entry.pdf_link       = row[3]
            m_entry.citation_count = row[4]
            m_entry.doi            = row[5]
            m_entry.journal        = row[6]
            m_entry.vol            = row[7]
            m_entry.page           = row[8]
            m_entry.pub_type       = row[9]
            m_entry.aps_id         = row[10]
            m_entry.aps_score      = row[11]
            m_entry.query_id       = row[12]

            # Query authors
            loop_cursor = self.db.cursor()
            loop_cursor.execute("""SELECT * FROM aps_author_association WHERE article_id="""+str(row[0]))

            for x in xrange(0, loop_cursor.rowcount):
                loop_row = loop_cursor.fetchone()

                # Add in all of the authors
                m_entry.authors.append(loop_row[1])

            # Query references
            loop_cursor = self.db.cursor()
            loop_cursor.execute("""SELECT * FROM aps_references WHERE article_id="""+str(row[0]))

            for x in xrange(0, loop_cursor.rowcount):
                loop_row = loop_cursor.fetchone()

                # Add in all of the authors
                m_entry.refs.append(loop_row[2])

            # Append the result converting to the correct datatype
            rows.append(m_entry)

        # Return the rows
        return rows

    def get_unique_journals_fac(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute("""SELECT DISTINCT journal FROM aps_article_metadata WHERE article_id in (SELECT DISTINCT article_id FROM aps_author_association WHERE """+query_str+""")""")

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()

            # Append the result converting to the correct datatype
            rows.append(row[0])

        # Return the rows
        return rows

    def get_article_entries_no_arrays(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute("""SELECT * FROM aps_article_metadata WHERE """+query_str)

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()

            # Initiailze a course catalog entry and set it
            m_entry = aps_article_entry()

            # Set all of the entries
            m_entry.article_id     = row[0]
            m_entry.title          = row[1]
            m_entry.pub_date       = row[2]
            m_entry.pdf_link       = row[3]
            m_entry.citation_count = row[4]
            m_entry.doi            = row[5]
            m_entry.journal        = row[6]
            m_entry.vol            = row[7]
            m_entry.page           = row[8]
            m_entry.pub_type       = row[9]
            m_entry.aps_id         = row[10]
            m_entry.aps_score      = row[11]
            m_entry.query_id       = row[12]

            # Append the result converting to the correct datatype
            rows.append(m_entry)

        # Return the rows
        return rows

    def get_unique_titles(self, query_str):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute("""SELECT DISTINCT title FROM course_catalog_faculty_entries WHERE """+query_str)

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()
            rows.append(row[0])

        # Return the rows
        return rows

    def get_course_catalog_unique_profs(self):
        # Get a cursor from the db object
        cursor = self.db.cursor()

        # Run the search query
        cursor.execute('select distinct(name) as name from course_catalog_faculty_entries order by name desc')

        # Iterate over the rows of our results adding them to an array (Shhh... my DB is small)
        rows = []
        numrows = cursor.rowcount
        for x in xrange(0,numrows):
            row = cursor.fetchone()
            rows.append(row[0])

        # Return the rows
        return rows

def create_connection(address, timeout=None, source_address=None):
    sock = socks.socksocket()
    sock.connect(address)
    return sock

def set_local_proxy():
    # Set our proxy
    socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, 'localhost', 8080)

    # patch the socket module
    socket.socket = socks.socksocket
    socket.create_connection = create_connection

Y = 2000 # dummy leap year to allow input X-02-29 (leap day)
seasons = [('winter', (datetime.date(Y,  1,  1),  datetime.date(Y,  3, 20))),
    ('spring', (datetime.date(Y,  3, 21),  datetime.date(Y,  6, 20))),
    ('summer', (datetime.date(Y,  6, 21),  datetime.date(Y,  9, 22))),
    ('autumn', (datetime.date(Y,  9, 23),  datetime.date(Y, 12, 20))),
    ('winter', (datetime.date(Y, 12, 21),  datetime.date(Y, 12, 31)))]

def get_season(date):
    date = date.replace(year=Y)
    for season in seasons:
        if date>= season[1][0] and date<= season[1][1]:
            return season[0]
    return 'Winter'

###############################################################################
#  GENERATE PLOTS
###############################################################################
# Graphs based only on faculty
def gen_year_dep_size(db):
    print 'Generating year_dep_size'
    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_course_catalog_entries('year(end) = '+str(x)+''' AND title NOT LIKE '%raduate%' ''')

        if(len(results) == 0):
            continue

        # Set the pair of results
        ret.append((x, len(results)))

    # Return the series
    return ret
    
def gen_year_title_dist(db):
    print 'Generating year_title_dist'
    # Make an empty list and a set of titles
    ret = []
    all_titles = set()

    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_course_catalog_entries('year(end) = '+str(x)+''' AND title NOT LIKE '%raduate%' ''')

        if(len(results) == 0):
            continue

        # Get the histogram of the titles
        m_titles = dict()
        for i in results:
            if i.title in m_titles:
                m_titles[i.title] += 1
            else:
                m_titles[i.title] = 1
                all_titles.add(i.title)

        # Convert them to percentages
        for i in m_titles:
            m_titles[i] = float(m_titles[i])/float(len(results))

        # Set the pair of results
        ret.append((x, m_titles))

    # Convert the set into a list
    all_titles = list(all_titles)
    
    # Instatiate an empty list to return and insert the first value as the titles
    final_list = []
    final_list.append(['index']+['"'+x+'"' for x in all_titles])


    # Iterate over all of the elements in ret converting the title dict into a set of percentages ranging over all titles
    for i in ret:
        # Instantiate an empty entry
        entry = []

        # Insert the year
        entry.append(i[0])

        # Iterate over the unique titles
        for j in all_titles:
            if (j in i[1]):
                entry.append(i[1][j])
            else:
                entry.append(0.0)

        final_list.append(entry)


    # Return the series
    return final_list

def gen_year_title_dist_mod(db):
    print 'Generating year_title_dist_mod'
    # Make an empty list and a set of titles
    ret = []
    all_titles = set()

    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_course_catalog_entries('year(end) = '+str(x)+''' AND title NOT LIKE '%raduate%' ''')

        if(len(results) == 0):
            continue

        # Get the histogram of the titles
        m_titles = dict()

        for i in results:
            # Convert the titles to a set of ranges
            # Associate Prof
            # Assistant Prof
            # Prof
            # Instructor
            m_title = ''

            if(i.title == 'Associate Professor' or i.title == 'Associate Professor, and Director of National Science Fdn. Summer Institute' or i.title == 'Associate Professor of Physics'):
                m_title = 'Associate Professor'
            elif(i.title == 'Assistant Professor' or i.title == 'Assistant Professor of Physics'):
                m_title = 'Assistant Professor'
            elif(i.title == 'Professor of Physics, and Director of Summer School and Evening Programs' or i.title == 'Head of Department' or i.title == 'Professor of Physics and Electrical Engineering' or i.title == 'Professor of Theoretical and Applied Mechanics' or i.title == 'Professor of Physics' or i.title == 'Professor of Applied Mechanics' or i.title == 'Professor' or i.title == 'Professor of Physics and Head of Department' or i.title == 'Professor of Higher Mathematics, Professor of Physics' or i.title == 'Professor of Mechanics and Physics' or i.title == 'Professor of Physics, Associate Dean of the Faculty, and Director of Summer School and Evening Programs'):
                m_title = 'Professor'
            else:
                m_title = 'Other'
                
        
            if m_title in m_titles:
                m_titles[m_title] += 1
            else:
                m_titles[m_title] = 1
                all_titles.add(m_title)

        # Convert them to percentages
        for i in m_titles:
            m_titles[i] = float(m_titles[i])/float(len(results))

        # Set the pair of results
        ret.append((x, m_titles))

    # Convert the set into a list
    all_titles = list(all_titles)
    
    # Instatiate an empty list to return and insert the first value as the titles
    final_list = []
    final_list.append(['index']+['"'+x+'"' for x in all_titles])


    # Iterate over all of the elements in ret converting the title dict into a set of percentages ranging over all titles
    for i in ret:
        # Instantiate an empty entry
        entry = []

        # Insert the year
        entry.append(i[0])

        # Iterate over the unique titles
        for j in all_titles:
            if (j in i[1]):
                entry.append(i[1][j])
            else:
                entry.append(0.0)

        final_list.append(entry)


    # Return the series
    return final_list

def gen_year_degree_dist(db):
    print 'Generating year_degree_dist'
    # Make an empty list and a set of titles
    ret = []
    all_titles = set()

    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_course_catalog_entries('year(end) = '+str(x)+''' AND title NOT LIKE '%raduate%' ''')

        if(len(results) == 0):
            continue

        # Get the histogram of the titles
        m_titles = dict()
        for i in results:
            if i.degree in m_titles:
                m_titles[i.degree] += 1
            else:
                m_titles[i.degree] = 1
                all_titles.add(i.degree)

        # Convert them to percentages
        for i in m_titles:
            m_titles[i] = float(m_titles[i])/float(len(results))

        # Set the pair of results
        ret.append((x, m_titles))

    # Convert the set into a list
    all_titles = list(all_titles)
    
    # Instatiate an empty list to return and insert the first value as the titles
    final_list = []
    final_list.append(['index']+['"'+x+'"' for x in all_titles])


    # Iterate over all of the elements in ret converting the title dict into a set of percentages ranging over all titles
    for i in ret:
        # Instantiate an empty entry
        entry = []

        # Insert the year
        entry.append(i[0])

        # Iterate over the unique titles
        for j in all_titles:
            if (j in i[1]):
                entry.append(i[1][j])
            else:
                entry.append(0.0)

        final_list.append(entry)


    # Return the series
    return final_list

def gen_year_degree_dist_mod(db):
    print 'Generating year_degree_dist_mod'
    # Make an empty list and a set of titles
    ret = []
    all_titles = set()

    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_course_catalog_entries('year(end) = '+str(x)+''' AND title NOT LIKE '%raduate%' ''')

        if(len(results) == 0):
            continue

        # Get the histogram of the titles
        m_titles = dict()
        for i in results:
            # Convert the degree into a range
            m_degree = ''

            if(i.degree == 'Sc.D.' or i.degree == 'Ph.D.' or i.degree == 'M.A., D.Sc., (Edin.)' or i.degree == 'D.Sc.'):
                m_degree = 'Doctoral'
            elif(i.degree == 'M.A.'or i.degree == 'M.E.' or i.degree == 'M.A., B.Sc., (Edin.)' or i.degree == 'A.M.' or i.degree == 'M.S.'):
                m_degree = 'Masters'
            elif(i.degree == 'Ph.B.' or i.degree == 'A.B.' or i.degree == 'B.S.' or i.degree == 'S.B.'):
                m_degree = 'Bachelors'
            else:
                m_degree = 'Other'


            if m_degree in m_titles:
                m_titles[m_degree] += 1
            else:
                m_titles[m_degree] = 1
                all_titles.add(m_degree)

        # Convert them to percentages
        for i in m_titles:
            m_titles[i] = float(m_titles[i])/float(len(results))

        # Set the pair of results
        ret.append((x, m_titles))

    # Convert the set into a list
    all_titles = list(all_titles)
    
    # Instatiate an empty list to return and insert the first value as the titles
    final_list = []
    final_list.append(['index']+['"'+x+'"' for x in all_titles])


    # Iterate over all of the elements in ret converting the title dict into a set of percentages ranging over all titles
    for i in ret:
        # Instantiate an empty entry
        entry = []

        # Insert the year
        entry.append(i[0])

        # Iterate over the unique titles
        for j in all_titles:
            if (j in i[1]):
                entry.append(i[1][j])
            else:
                entry.append(0.0)

        final_list.append(entry)


    # Return the series
    return final_list

def gen_course_catalog_tenure_bins(self):
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the course catalog entries associated w/ it
        ten = len(self.get_course_catalog_entries("name='"+self.db.escape_string(name)+"'"+" AND title NOT LIKE '%meritus%'"))
        if(ten in bins):
            bins[ten] += 1
        else:
            bins[ten] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    return ret

def avg_course_catalog_tenure(self):
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the course catalog entries associated w/ it
        ten = len(self.get_course_catalog_entries("name='"+self.db.escape_string(name)+"'"+" AND title NOT LIKE '%meritus%'"))
        if(ten in bins):
            bins[ten] += 1
        else:
            bins[ten] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    # Get the average of the tenure lengths
    avg = 0.0
    num_samples = 0
    for i in ret:
        avg += i[0]*i[1]
        num_samples += i[1]
    avg /= num_samples

    return avg

def gen_bins_num_titles(self):
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the course catalog entries associated w/ it
        sample = len(db.get_unique_titles("name='"+self.db.escape_string(name)+"'"))
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    return ret

def avg_num_titles(self):
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the course catalog entries associated w/ it
        sample = len(db.get_unique_titles("name='"+self.db.escape_string(name)+"'"))
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))
        ret.append((i, bins[i]))

    # Get the average
    avg = 0.0
    num_samples = 0
    for i in ret:
        avg += i[0]*i[1]
        num_samples += i[1]
    avg /= num_samples

    return avg

###############################################################################
# Graphs based on articles
def gen_year_num_pubs(db):
    print 'Generating year_num_pubs'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        #if(len(results) == 0):
        #    continue

        # Set the pair of results
        ret.append((x, len(results)))

    # Return the series
    return ret
    
def gen_year_avg_auths(db):
    print 'Generating year_avg_auths'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        avg = 0

        if(len(results) != 0):
            # Average the number of authors
            total = 0

            for i in results:
                total += len(i.authors)

            avg = float(total)/float(len(results))

        # Set the pair of results
        ret.append((x, avg))

    # Return the series
    return ret

def gen_year_avg_refs(db):
    print 'Generating year_avg_refs'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        avg = 0

        if(len(results) != 0):
            # Average the number of authors
            total = 0

            for i in results:
                total += len(i.refs)

            avg = float(total)/float(len(results))

        # Set the pair of results
        ret.append((x, avg))

    # Return the series
    return ret

def gen_year_avg_citations(db):
    print 'Generating year_avg_citations'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        avg = 0

        if(len(results) != 0):
            # Average the number of authors
            total = 0

            for i in results:
                total += i.citation_count

            avg = float(total)/float(len(results))

        # Set the pair of results
        ret.append((x, avg))

    # Return the series
    return ret

def gen_year_journal_dist(db):
    print 'Generating year_journal_dist'
    # Make an empty list and a set of titles
    ret = []
    all_titles = set()

    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        if(len(results) == 0):
            continue

        # Get the histogram of the titles
        m_titles = dict()
        for i in results:
            if i.journal in m_titles:
                m_titles[i.journal] += 1
            else:
                m_titles[i.journal] = 1
                all_titles.add(i.journal)

        # Convert them to percentages
        for i in m_titles:
            m_titles[i] = float(m_titles[i])/float(len(results))

        # Set the pair of results
        ret.append((x, m_titles))

    # Convert the set into a list
    all_titles = list(all_titles)
    
    # Instatiate an empty list to return and insert the first value as the titles
    final_list = []
    final_list.append(['index']+['"'+x+'"' for x in all_titles])


    # Iterate over all of the elements in ret converting the title dict into a set of percentages ranging over all titles
    for i in ret:
        # Instantiate an empty entry
        entry = []

        # Insert the year
        entry.append(i[0])

        # Iterate over the unique titles
        for j in all_titles:
            if (j in i[1]):
                entry.append(i[1][j])
            else:
                entry.append(0.0)

        final_list.append(entry)


    # Return the series
    return final_list

def gen_year_pub_season(db):
    print 'Generating year_pub_season'
    # Make an empty list and a set of titles
    ret = []
    all_titles = set()

    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        if(len(results) == 0):
            continue

        # Get the histogram of the titles
        m_titles = dict()
        for i in results:
            m_season = get_season(i.pub_date)
            if m_season in m_titles:
                m_titles[m_season] += 1
            else:
                m_titles[m_season] = 1
                all_titles.add(m_season)

        # Convert them to percentages
        for i in m_titles:
            m_titles[i] = float(m_titles[i])/float(len(results))

        # Set the pair of results
        ret.append((x, m_titles))

    # Convert the set into a list
    all_titles = list(all_titles)
    
    # Instatiate an empty list to return and insert the first value as the titles
    final_list = []
    final_list.append(['index']+['"'+x+'"' for x in all_titles])


    # Iterate over all of the elements in ret converting the title dict into a set of percentages ranging over all titles
    for i in ret:
        # Instantiate an empty entry
        entry = []

        # Insert the year
        entry.append(i[0])

        # Iterate over the unique titles
        for j in all_titles:
            if (j in i[1]):
                entry.append(i[1][j])
            else:
                entry.append(0.0)

        final_list.append(entry)


    # Return the series
    return final_list

def gen_year_avg_pub_date(db):
    print 'Generating year_avg_pub_date'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        avg = 0

        if(len(results) != 0):
            # Average the number of authors
            total = 0

            for i in results:
                total += (i.pub_date - datetime.date(x, 01, 01)).days

            avg = float(total)/float(len(results))

        # Set the pair of results
        ret.append((x, avg))

    # Return the series
    return ret

def gen_year_std_auths(db):
    print 'Generating year_var_auths'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        if(len(results) <= 1):
            continue
        
        dist = [len(i.authors) for i in results]

        # Set the pair of results
        ret.append((x, stdev(dist)))

    # Return the series
    return ret

def gen_year_std_refs(db):
    print 'Generating year_var_refs'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        if(len(results) <= 1):
            continue
        
        dist = [len(i.refs) for i in results]

        # Set the pair of results
        ret.append((x, stdev(dist)))

    # Return the series
    return ret

def gen_year_std_citations(db):
    print 'Generating year_var_citations'

    # Make an empty list
    ret = []
 
    # Iterate over each year in the study
    for x in xrange(start_year, end_year+1):
        # Get the entries for this year excluding grad students
        results = db.get_article_entries('year(pub_date) = '+str(x))

        if(len(results) <= 1):
            continue
        
        dist = [i.citation_count for i in results]

        # Set the pair of results
        ret.append((x, stdev(dist)))

    # Return the series
    return ret

def gen_bins_citation_count(db):
    bins = dict()

    # Get all of the articles
    articles = db.get_article_entries_no_arrays("'a'='a'")

    # Iterate over the them 
    for i in articles:
        # Get the course catalog entries associated w/ it
        sample = i.citation_count
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    return ret

def gen_bins_day(db):
    bins = dict()

    # Get all of the articles
    articles = db.get_article_entries_no_arrays("'a'='a'")

    # Iterate over the them 
    for i in articles:
        # Get the course catalog entries associated w/ it
        sample = (i.pub_date - datetime.date(i.pub_date.year, 01, 01)).days
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    return ret

def gen_bins_refs(db):
    bins = dict()

    # Get all of the articles
    articles = db.get_article_entries("'a'='a'")

    # Iterate over the them 
    for i in articles:
        # Get the course catalog entries associated w/ it
        sample = len(i.refs)
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    return ret

def gen_bins_auths(db):
    bins = dict()

    # Get all of the articles
    articles = db.get_article_entries("'a'='a'")

    # Iterate over the them 
    for i in articles:
        # Get the course catalog entries associated w/ it
        sample = len(i.authors)
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        ret.append((i, bins[i]))

    return ret

###############################################################################
# Graphs combining articles and authors
def gen_bins_pubs_per_auth(self):
    print 'Generating bins_pubs_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = len(db.get_article_entries_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"))
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i != 0):
            ret.append((i, bins[i]))

    return ret

def avg_pubs_per_auth(self):
    print 'Generating bins_pubs_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = len(db.get_article_entries_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"))
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i != 0):
            ret.append((i, bins[i]))

    # Get the average
    avg = 0.0
    num_samples = 0
    for i in ret:
        avg += i[0]*i[1]
        num_samples += i[1]
    avg /= num_samples

    return avg

def gen_bins_citations_per_auth(self):
    print 'Generating bins_citations_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = 0
        for i in db.get_article_entries_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"):
            sample += i.citation_count

        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i != 0):
            ret.append((i, bins[i]))

    return ret

def avg_citations_per_auth(self):
    print 'Generating bins_citations_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = 0
        for i in db.get_article_entries_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"):
            sample += i.citation_count

        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i != 0):
            ret.append((i, bins[i]))

    # Get the average
    avg = 0.0
    num_samples = 0
    for i in ret:
        avg += i[0]*i[1]
        num_samples += i[1]
    avg /= num_samples

    return avg

def gen_bins_refs_per_auth(self):
    print 'Generating bins_refs_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = 0
        for i in db.get_article_entries_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"):
            sample += len(i.refs)

        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i!=0):
            ret.append((i, bins[i]))

    return ret

def avg_refs_per_auth(self):
    print 'Generating bins_refs_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = 0
        for i in db.get_article_entries_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"):
            sample += len(i.refs)

        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i!=0):
            ret.append((i, bins[i]))

    # Get the average
    avg = 0.0
    num_samples = 0
    for i in ret:
        avg += i[0]*i[1]
        num_samples += i[1]
    avg /= num_samples

    return avg

def gen_bins_unique_journals_per_auth(self):
    print 'Generating bins_unique_journals_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = len(db.get_unique_journals_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"))
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i!=0):
            ret.append((i, bins[i]))

    return ret

def avg_unique_journals_per_auth(self):
    print 'Generating bins_unique_journals_per_auth'
    # Get the unique prof names
    names = self.get_course_catalog_unique_profs()

    bins = dict()

    # Iterate over the names
    for name in names:
        # Get the last name
        hname = HumanName(name)
        lastname = hname.last 

        # Hack to get humanname to work
        if "Bluemel" in name:
            lastname = "Bluemel"

        if "-----" in name:
            continue

        # Get the course catalog entries associated w/ it
        sample = len(db.get_unique_journals_fac("name LIKE '%"+self.db.escape_string(lastname)+"%'"))
        if(sample in bins):
            bins[sample] += 1
        else:
            bins[sample] = 1

    # Turn the dict into an array
    ret = []
    for i in bins:
        if(i!=0):
            ret.append((i, bins[i]))

    # Get the average
    avg = 0.0
    num_samples = 0
    for i in ret:
        avg += i[0]*i[1]
        num_samples += i[1]
    avg /= num_samples

    return avg

###############################################################################
# PROGRAM ENTRY POINT
###############################################################################

# Seed the random number generator
random.seed()

# Set a proxy
# set_local_proxy()

# Connect to the database
db = database_interface()
db.connect()

print("average tenure: "+str(avg_course_catalog_tenure(db)))
print("average number of titles: "+str(avg_num_titles(db)))
print("average publications in career: "+str(avg_pubs_per_auth(db)))
print("average citations in career: "+str(avg_citations_per_auth(db)))
print("average references in career: "+str(avg_refs_per_auth(db)))
print("average journals in career: "+str(avg_unique_journals_per_auth(db)))

# Generate some graphs
output_gnuplot(output_dir+'/dep_size.dat', gen_year_dep_size(db))
output_gnuplot(output_dir+'/dep_titles.dat', gen_year_title_dist_mod(db))
output_gnuplot(output_dir+'/dep_degrees.dat', gen_year_degree_dist_mod(db))
output_gnuplot(output_dir+'/num_pubs.dat', gen_year_num_pubs(db))
output_gnuplot(output_dir+'/avg_auths.dat', gen_year_avg_auths(db))
output_gnuplot(output_dir+'/avg_refs.dat', gen_year_avg_refs(db))
output_gnuplot(output_dir+'/journal_dist.dat', gen_year_journal_dist(db))
output_gnuplot(output_dir+'/avg_citations.dat', gen_year_avg_citations(db))
output_gnuplot(output_dir+'/pub_season.dat', gen_year_pub_season(db))
output_gnuplot(output_dir+'/avg_pub_date.dat', gen_year_avg_pub_date(db))
output_gnuplot(output_dir+'/std_auths.dat', gen_year_std_auths(db))
output_gnuplot(output_dir+'/std_refs.dat', gen_year_std_refs(db))
output_gnuplot(output_dir+'/std_citations.dat', gen_year_std_citations(db))
output_gnuplot(output_dir+'/tenure_histo.dat', gen_course_catalog_tenure_bins(db))
output_gnuplot(output_dir+'/citation_histo.dat', gen_bins_citation_count(db))
output_gnuplot(output_dir+'/day_histo.dat', gen_bins_day(db))
output_gnuplot(output_dir+'/refs_histo.dat', gen_bins_refs(db))
output_gnuplot(output_dir+'/auths_histo.dat', gen_bins_auths(db))
output_gnuplot(output_dir+'/num_titles_histo.dat', gen_bins_num_titles(db))
output_gnuplot(output_dir+'/pubs_per_auth_histo.dat', gen_bins_pubs_per_auth(db))
output_gnuplot(output_dir+'/citations_per_auth_histo.dat', gen_bins_citations_per_auth(db))
output_gnuplot(output_dir+'/refs_per_auth_histo.dat', gen_bins_refs_per_auth(db))
output_gnuplot(output_dir+'/journals_per_auth_histo.dat', gen_bins_unique_journals_per_auth(db))
